import { defineStore } from "pinia"
import { ref } from "vue"
import { request } from "@/scripts/request"
import { Preferences } from "@capacitor/preferences"
import { useRealms } from "@/stores/realms"
import { useChannels } from "@/stores/channels"

export interface Userinfo {
  isReady: boolean
  isLoggedIn: boolean
  displayName: string
  idSet: { [id: string]: number }
  data: any
}

const defaultUserinfo: Userinfo = {
  isReady: false,
  isLoggedIn: false,
  displayName: "Citizen",
  idSet: {},
  data: null
}

export async function getAtk() {
  return (await Preferences.get({ key: "identity.access_token" })).value
}

export async function getRtk() {
  return (await Preferences.get({ key: "identity.refresh_token" })).value
}

export async function checkLoggedIn(): Promise<boolean> {
  return (await Preferences.get({ key: "identity.access_token" })).value != null
}

export const useUserinfo = defineStore("userinfo", () => {
  const userinfoHooks = {
    after: [useRealms().list, useChannels().list, useChannels().connect]
  }

  const userinfo = ref(defaultUserinfo)
  const isReady = ref(false)

  async function readProfiles() {
    if (!(await checkLoggedIn())) {
      isReady.value = true
    }

    const res = await request("identity", "/api/users/me", {
      headers: { Authorization: `Bearer ${await getAtk()}` }
    })
    if (res.status !== 200) {
      return
    }

    const data = await res.json()

    const federationResp = await Promise.all([
      request("interactive", "/api/users/me", {
        headers: { Authorization: `Bearer ${await getAtk()}` }
      }),
      request("messaging", "/api/users/me", {
        headers: { Authorization: `Bearer ${await getAtk()}` }
      })
    ])

    userinfo.value = {
      isReady: true,
      isLoggedIn: true,
      displayName: data["nick"],
      idSet: {
        interactive: (await federationResp[0].json())["id"],
        messaging: (await federationResp[1].json())["id"]
      },
      data: data
    }

    userinfoHooks.after.forEach((call) => call())
  }

  return { userinfo, isReady, readProfiles }
})
